Package: BAS
Version: 1.6.2
Date: 2022-04-23
Title: Bayesian Variable Selection and Model Averaging using Bayesian
        Adaptive Sampling
Authors@R: c(person("Merlise", "Clyde", email="clyde@duke.edu",
	   		       role=c("aut","cre", "cph"),
	   		       comment=c("ORCID=0000-0002-3595-1872")
	   		       ),
           person("Michael", "Littman", role="ctb"),
	   person("Quanli", "Wang", role="ctb"),
	   person("Joyee", "Ghosh", role="ctb"),
	   person("Yingbo", "Li", role="ctb"),
	   person("Don", "van de Bergh", role="ctb"))
Depends: R (>= 3.0)
Imports: stats, graphics, utils, grDevices
Suggests: MASS, knitr, ggplot2, GGally, rmarkdown, roxygen2, dplyr,
        glmbb, testthat, covr
Description: Package for Bayesian Variable Selection and  Model Averaging 
    in linear models and generalized linear models using stochastic or
    deterministic sampling without replacement from posterior
    distributions.  Prior distributions on coefficients are
    from Zellner's g-prior or mixtures of g-priors
    corresponding to the Zellner-Siow Cauchy Priors or the
    mixture of g-priors from Liang et al (2008)
    <DOI:10.1198/016214507000001337>
    for linear models or mixtures of g-priors from  Li and Clyde
    (2019) <DOI:10.1080/01621459.2018.1469992> in generalized linear models.
    Other model selection criteria include AIC, BIC and Empirical Bayes 
    estimates of g. Sampling probabilities may be updated based on the sampled
    models using sampling w/out replacement or an efficient MCMC algorithm which
    samples models using a tree structure of the model space 
    as an efficient hash table.  See  Clyde, Ghosh and Littman (2010) 
    <DOI:10.1198/jcgs.2010.09049> for  details on the sampling algorithms.
    Uniform priors over all models or beta-binomial prior distributions on
    model size are allowed, and for large p truncated priors on the model
    space may be used to enforce sampling models that are full rank.  
    The user may force variables to always be included in addition to imposing
    constraints that higher order interactions are included only if their 
    parents are included in the model.
    This material is based upon work supported by the National Science
    Foundation under Division of Mathematical Sciences grant 1106891.
    Any opinions, findings, and
    conclusions or recommendations expressed in this material are those of
    the author(s) and do not necessarily reflect the views of the
    National Science Foundation.
License: GPL (>= 3)
URL: https://www.r-project.org, https://github.com/merliseclyde/BAS
BugReports: https://github.com/merliseclyde/BAS/issues
Repository: CRAN
NeedsCompilation: yes
ByteCompile: yes
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.1.2
Packaged: 2022-04-23 15:52:56 UTC; clyde
Author: Merlise Clyde [aut, cre, cph] (ORCID=0000-0002-3595-1872),
  Michael Littman [ctb],
  Quanli Wang [ctb],
  Joyee Ghosh [ctb],
  Yingbo Li [ctb],
  Don van de Bergh [ctb]
Maintainer: Merlise Clyde <clyde@duke.edu>
Date/Publication: 2022-04-26 07:30:06 UTC
Built: R 4.0.5; x86_64-w64-mingw32; 2022-04-27 14:23:22 UTC; windows
Archs: x64
