# Generated by roxygen2: do not edit by hand

S3method(coef,bas)
S3method(confint,coef.bas)
S3method(confint,pred.bas)
S3method(fitted,bas)
S3method(image,bas)
S3method(plot,bas)
S3method(plot,coef.bas)
S3method(plot,confint.bas)
S3method(predict,bas)
S3method(predict,basglm)
S3method(print,bas)
S3method(print,coef.bas)
S3method(summary,bas)
S3method(update,bas)
S3method(variable.names,pred.bas)
export(Bayes.outlier)
export(Bernoulli)
export(Bernoulli.heredity)
export(CCH)
export(EB.global)
export(EB.local)
export(IC.prior)
export(Jeffreys)
export(TG)
export(aic.prior)
export(bas.glm)
export(bas.lm)
export(bayesglm.fit)
export(beta.binomial)
export(beta.prime)
export(bic.prior)
export(cv.summary.bas)
export(diagnostics)
export(eplogprob)
export(eplogprob.marg)
export(force.heredity.bas)
export(g.prior)
export(hyper.g)
export(hyper.g.n)
export(hypergeometric1F1)
export(hypergeometric2F1)
export(intrinsic)
export(list2matrix.bas)
export(list2matrix.which)
export(phi1)
export(robust)
export(tCCH)
export(testBF.prior)
export(tr.beta.binomial)
export(tr.poisson)
export(tr.power.prior)
export(uniform)
export(which.matrix)
import(grDevices)
import(graphics)
import(stats)
useDynLib(BAS, .registration=TRUE, .fixes="C_")
